@@ -1,10 +1,10 @@
 package Mojolicious::Plugin::RenderCGI;
 
 use Mojo::Base 'Mojolicious::Plugin';
-#~ use Capture::Tiny qw(capture);
 use Mojolicious::Plugin::RenderCGI::CGI;
+use Mojo::Util qw(encode md5_sum);
 
-our $VERSION = '0.001';
+our $VERSION = '0.01';
 my $pkg = __PACKAGE__;
 my %cache = ();
 
@@ -14,87 +14,89 @@ sub register {
   $conf->{import} ||= [qw(:html :form)];
   $conf->{import} = [grep /\w/, split(/\s+/, $conf->{import})]
     unless ref $conf->{import};
-  $conf->{skip_fatals} //= $app->mode ne 'development' ? 1 : 0;
+  #~ $conf->{skip_fatals} //= $app->mode eq 'development' ? 0 : 1;
+  $conf->{exception} //= {'handler'=>'ep', 'layout' => undef,};
+  #~ $conf->{not_found} //= $app->mode eq 'development' ? 'template' : 'exception';
     
-  my $renderer = Mojolicious::Plugin::RenderCGI::CGI->new(
+  my $cgi = Mojolicious::Plugin::RenderCGI::CGI->new(
     import=>$conf->{import},
   );
   
+  my $enough = sub {# харе
+    my ($error, $c) = @_;
+    $c->stash(%{$conf->{exception}})
+      and die $error
+      if ref $conf->{exception} eq 'HASH';
+    
+    $app->log->error($error);# лог после die!
+    return $conf->{exception} eq 'template' ? $error : '';
+  };
+  
   $app->renderer->add_handler(
     cgi => sub {
       my ($r, $c, $output, $options) = @_;
       #~ $app->log->debug($app->dumper($options));
       
-      
       # относительный путь шаблона
-      my $name = $r->template_name($options);
+      my $content = $options->{inline};# встроенный шаблон
+      my $name = defined $content ? md5_sum encode('UTF-8', $content) : undef;
+      return unless defined($name //= $r->template_name($options));
+      
+      my ($rend, $from) = ($cache{$name}, 'cache');# подходящий шаблон из кэша 
       
       my $stash = $c->stash($pkg);
       $c->stash($pkg => {stack => []})
         unless $stash;
       $stash ||= $c->stash($pkg);
       my $last_template = $stash->{stack}[-1];
-      die "Loops template [$name]!"
-        if $last_template && $last_template eq $name;
+      if ($last_template && $last_template eq $name) {
+        $$output = $enough->("Stop looping template [$name]!", $c);
+        return;
+      }
       push @{$stash->{stack}}, $name;
-
-      # встроенный шаблон
-      my $content = $options->{inline};
       
-      my ($error, $rend, $from) = (undef, undef, 'inline');
-      
-      #~ $$output = '';
-      unless (defined $content) {# не inline
-        # подходящий шаблон из кэша но не inline
-        ($rend, $from) = ($cache{$name}, 'cache');
-        
-        unless ($rend) {# не кэш
+      unless ($rend) {#не кэш
+        if (defined $content) {# инлайн
+          $from = 'inline';
+        } else {
           # подходящий шаблон в секции DATA
           ($content, $from) = ($r->get_data_template($options), 'DATA section');#,, $name
           
-          unless (defined $content) {
+          unless (defined $content) {# file
           #  абсолютный путь шаблона
             if (my $path = $r->template_path($options)) {
               my $file = Mojo::Asset::File->new(path => $path);
               ($content, $from) = ($file->slurp, 'file');
               
             } else {
-              $error = sprintf(qq{Template "%s" does not found}, $name);
-              $$output = $conf->{skip_fatals} ? '' : $error;
-              $app->log->error($error);
+              $$output = $enough->(sprintf(qq{Template "%s" does not found}, $name), $c);
               return;
             }
           }
         }
       }
       
-      ($$output = '')
-        or $app->log->debug(sprintf(qq{Empty template "%s"}, $name))
+      $$output = '';
+      $app->log->debug(sprintf(qq{Empty template "%s"}, $name))
         and return
         unless $rend || defined($content) && $content !~ /^\s*$/;
       
-      $rend ||= $renderer->compile($content)
-        or ($error = sprintf(qq{Template "%s" does not exists}, $name // $from))
-        and (($$output = $conf->{skip_fatals} ? '' : $error) || 1)
-        and $app->log->error($error)
-        and return;
-      
-      ($error = sprintf(qq{Compile error "%s": %s}, $name // $from, $rend))
-        and (($$output = $conf->{skip_fatals} ? '' : $error) || 1)
-        and $app->log->error($error)
-        and return
-        unless ref $rend eq 'CODE';
+      $rend ||= $cgi->compile($content);
+
+      unless (ref $rend eq 'CODE') {
+        $$output = $enough->(sprintf(qq{Compile error template "%s": %s}, $name // $from, $rend), $c);
+        return;
+      }
 
       $app->log->debug(sprintf(qq{Rendering template "%s" from the %s}, $name, $from,));
       $cache{$name} ||= $rend;
-      # Передать rendered результат обратно в рендерер
-      my @out = eval { $rend->($c, $renderer->{cgi},)};
       
-      ($error = sprintf(qq{Die on "%s":\n%s}, $name // $from, $@))
-        and (($$output = $conf->{skip_fatals} ? '' : $error) || 1)
-        and $app->log->error($error)
-        and return
-        if $@;
+      # Передать rendered результат обратно в рендерер
+      my @out = eval { $rend->($c, $cgi->{cgi})};
+      if ($@) {
+        $$output = $enough->(sprintf(qq{Die on template "%s":\n%s}, $name // $from, $@), $c);
+        return;
+      }
       
       $$output = join"\n", grep defined, @out;
       
@@ -103,7 +105,6 @@ sub register {
 }
 
 
-
 1;
 
 
@@ -119,11 +120,11 @@ sub register {
 
 =head1 VERSION
 
-0.001
+0.01
 
 =head1 NAME
 
-Mojolicious::Plugin::RenderCGI - Rendering template with Perl code and CGI.pm subs imports.
+Mojolicious::Plugin::RenderCGI - Rendering template with Perl code and CGI.pm funcs/subs for tags emits.
 
 =head1 SYNOPSIS
 
@@ -140,7 +141,7 @@ Mojolicious::Plugin::RenderCGI - Rendering template with Perl code and CGI.pm su
 
 =head1 Template
 
-File name like "templates/foo/bar.html.cgi"
+Template is a Perl code that generate content as list of statements. Similar to C<do> BLOCK. Template file name like "templates/foo/bar.html.cgi"
 
   # $c and $self already are controller
   # $cgi is a CGI object (OO-style)
@@ -148,20 +149,30 @@ File name like "templates/foo/bar.html.cgi"
   $c->layout('default', handler=>'ep',);# set handler 'ep' for all includes !!!
   my $foo = $c->stash('foo')
     or die "Where is your FOO?";
+  
   #=======================================
   #======= content comma list! ===========
   #=======================================
-  $c->include('far', handler=>'cgi'),# change handler against layout
-  $c->include('bax'); # handler still "cgi" unless template "bax" (and its includes) didn`t changed it
+  
   h1({}, "Welcome"),
+  
+  $c->include('foo', handler=>'cgi'),# change handler against layout
+  $c->include('bar'); # handler still "cgi" unless template "foo" (and its includes) didn`t changes it
+  
   <<END_HTML,
-  <input id="bah" name="bah" type="checkbox" />
-  <label for="bah">$foo</label>
+  <!-- comment -->
   END_HTML
+  
   $self->app->log->info("Template has done")
     && undef,
 
-There are NO Mojolicious helpers without OO-style: B<$c-\>> OR b<$self-\>> prefix.
+There are NO Mojolicious helpers without OO-style: B<$c->> OR B<$self->> prefix.
+
+B<REMEMBER!> Escapes untrusted data. No auto escapes!
+
+  div({}, esc(...UNTRUSTED DATA...)),
+
+C<esc> is a shortcut for &CGI::escapeHTML.
 
 =head1 Options
 
@@ -178,12 +189,17 @@ Default is ':html :form' (string) same as [qw(:html :form)] (arrayref).
 
   import=>[], # none import, CGI OO-style only
 
-=head2 skip_fatals (bool)
+=head2 exception (string|hashref)
 
-Show fatal errors (not found, compile and runtime errors) as content of there template.
-By default on development mode set to 0 and 1 on production. Works on cgi handler only.
+To show fatal errors (not found, compile and runtime errors) as content of there template you must set string B<template>.
 
-  skip_fatals=>1, 
+To show fatals as standard Mojolicious 'exception.<mode>.html.ep' page  - set hashref like {'handler'=>'ep', 'layout' => undef,}.
+
+Overwise fatals are skips (empty string whole template).
+
+By default set to hashref {'handler'=>'ep', 'layout' => undef,}.
+
+  fatals=>'template', 
 
 =head1 Methods, subs, helpers...
 
@@ -193,6 +209,8 @@ Implements register method only. Register new renderer handler 'cgi'. No new hel
 
 L<CGI>
 
+L<CGI::HTML::Functions>
+
 L<Mojolicious::Plugin::TagHelpers>
 
 L<HTML::Tiny>
