@@ -1,5 +1,6 @@
 package Mojolicious::Plugin::RenderCGI::CGI;
 use Mojo::Base -strict;
+#~ use Capture::Tiny qw(capture);
 
 sub new {
     my $proto = shift;
@@ -13,9 +14,9 @@ sub new {
 sub init {
   my $self = shift;
   my %arg = @_;
+  push @{$arg{import}}, 'escapeHTML';
   require CGI;
-  CGI->import(@{$arg{import}})
-    if @{$arg{import}};
+  CGI->import(@{$arg{import}});
   $self->{cgi} = CGI->new();
   return $self;
 }
@@ -33,9 +34,11 @@ my \$cgi = shift;
 $code
 }
 CODE
-  return qq{Ошибка компиляции шаблона:\n$@}
+  return $@
     if $@;
   return $sub;
 }
 
+sub esc { escapeHTML(@_) }
+
 1;
\ No newline at end of file
